@@ -36,7 +36,7 @@ BEGIN {
 ##======================================================================
 ## globals
 
-our $VERSION = "0.01.010";
+our $VERSION = "0.01.011";
 our @ISA  = qw(DiaColloDB::Logger);
 
 ##======================================================================
@@ -455,7 +455,7 @@ sub requestQuery { return $_[0]{request_query}; }
 sub httpReferer { return $_[0]{http_referer}; }
 sub httpHost { return $_[0]{http_host}; }
 sub serverAddr { return $_[0]{server_addr}; }
-sub serverPort { return $_[0]{server_port} || 80; }
+sub serverPort { return $_[0]{server_port} || ($ENV{HTTPS} ? 443 : 80); }
 
 ## $uri    = $dbcgi->uri()
 ## $uri    = $dbcgi->uri($uri)
@@ -468,7 +468,7 @@ sub uri {
   return URI->new(
 		  #($host ? "http://$host" : "file://")
 		  ($host ? "${scheme}://$host" : "file://") ##-- guess scheme from HTTPS environment variable
-		  .($port==80 ? '' : ":$port")
+		  .($port==($scheme eq 'https' ? 443 : 80) ? '' : ":$port")
 		  .$dbcgi->requestUri
 		 );
 }
