@@ -541,8 +541,8 @@ function dcpParseFlat(data,opts) {
 				  score:[], value:[], avalue:[], sizes:[], opacity:[]
 				 });
 		if (isDiff) {
-		    idata.N1 = ffmt(p.prf1.N);
-		    idata.N2 = ffmt(p.prf2.N);
+		    idata.N1 = []; //ffmt(p.prf1.N);
+		    idata.N2 = []; //ffmt(p.prf2.N);
 		    idata.af1 = [];
 		    idata.bf1 = [];
 		    idata.af2 = [];
@@ -552,7 +552,7 @@ function dcpParseFlat(data,opts) {
 		    idata.ascore=[];
 		    idata.bscore=[];
 		} else {
-		    idata.N   = ffmt(p.N);
+		    idata.N   = []; //ffmt(p.N);
 		    idata.f1  = [];
 		    idata.f2  = [];
 		    idata.f12 = [];
@@ -566,6 +566,8 @@ function dcpParseFlat(data,opts) {
 	    idata.opacity[pi] = 1;
 
 	    if (isDiff) {
+		idata.N1[pi] = ffmt(p.prf1.N);
+		idata.N2[pi] = ffmt(p.prf2.N);
 		idata.af1[pi] = ffmt(p.prf1.f1);
 		idata.bf1[pi] = ffmt(p.prf2.f1);
 		idata.af2[pi] = ffmt(p.prf1.f2[item]);
@@ -575,6 +577,7 @@ function dcpParseFlat(data,opts) {
 		idata.ascore[pi] = sfmt(p.prf1[p.score][item]);
 		idata.bscore[pi] = sfmt(p.prf2[p.score][item]);
 	    } else {
+		idata.N[pi]   = ffmt(p.N);
 		idata.f1[pi]  = ffmt(p.f1);
 		idata.f2[pi]  = ffmt(p.f2[item]);
 		idata.f12[pi] = ffmt(p.f12[item]);
@@ -739,7 +742,7 @@ function d3InfoPopup(d,opts) {
 		    + '</td></tr>'
 		    +'<tr><th>details:</th><td class="diff details">'+(
 			'<table>'
-			    +'<tr><th>N(a/b):</th><td class="num N1">'+d.N1+'</td><td>/</td><td class="num N2">'+d.N2+'</tr>'
+			    +'<tr><th>N(a/b):</th><td class="num N1">'+d.N1[dsnap]+'</td><td>/</td><td class="num N2">'+d.N2[dsnap]+'</tr>'
 			    +'<tr><th>f1(a/b):</th><td class="num af1">'+d.af1[dsnap]+'</td><td>/</td><td class="num bf1">'+d.bf1[dsnap]+'</tr>'
 			    +'<tr><th>f2(a/b):</th><td class="num af2">'+d.af2[dsnap]+'</td><td>/</td><td class="num bf2">'+d.bf2[dsnap]+'</tr>'
 			    +'<tr><th>f12(a/b):</th><td class="num af12">'+d.af12[dsnap]+'</td><td>/</td><td class="num bf12">'+d.bf12[dsnap]+'</tr>'
@@ -758,7 +761,7 @@ function d3InfoPopup(d,opts) {
 		    + '</td></tr>'
 		    +'<tr><th>details:</th><td class="details">'+(
 			'<table>'
-			    +'<tr><th>N:</th><td class="num N">'+d.N+'</td></tr>'
+			    +'<tr><th>N:</th><td class="num N">'+d.N[dsnap]+'</td></tr>'
 			    +'<tr><th>f1:</th><td class="num f1">'+d.f1[dsnap]+'</td></tr>'
 			    +'<tr><th>f2:</th><td class="num f2">'+d.f2[dsnap]+'</td></tr>'
 			    +'<tr><th>f12:</th><td class="num f12">'+d.f12[dsnap]+'</td></tr>'
@@ -787,8 +790,8 @@ function d3InfoPopupUpdate(snapto) {
 	if (d3InfoCur.kwic != null)  { d3InfoCur.kwic.tr[0]  = dkeys[snapto]; dlg.find(".kwic").prop('href',kwicurl(d3InfoCur.kwic)); }
 	if (d3InfoCur.bkwic != null) { d3InfoCur.bkwic.tr[0] = dkeys[snapto]; dlg.find(".bkwic").prop('href',kwicurl(d3InfoCur.bkwic)); }
 	if (isDiff) {
-	    dlg.find(".N1").text(String(d.N1));
-	    dlg.find(".N2").text(String(d.N2));
+	    dlg.find(".N1").text(String(d.N1[snapto]));
+	    dlg.find(".N2").text(String(d.N2[snapto]));
 	    dlg.find(".af1").text(String(d.af1[snapto]));
 	    dlg.find(".bf1").text(String(d.bf1[snapto]));
 	    dlg.find(".af2").text(String(d.af2[snapto]));
@@ -798,7 +801,7 @@ function d3InfoPopupUpdate(snapto) {
 	    dlg.find(".ascore").text(String(d.ascore[snapto]));
 	    dlg.find(".bscore").text(String(d.bscore[snapto]));
 	} else {
-	    dlg.find(".N").text(String(d.N));
+	    dlg.find(".N").text(String(d.N[snapto]));
 	    dlg.find(".f1").text(String(d.f1[snapto]));
 	    dlg.find(".f2").text(String(d.f2[snapto]));
 	    dlg.find(".f12").text(String(d.f12[snapto]));
@@ -828,17 +831,17 @@ function dcpBrushSlider(dlabels, selector, opts) {
     //-- common variables
     var width  = opts.width - margin.left - margin.right;
     var height = opts.height - margin.top - margin.bottom; 
-    var xscale = d3.scale.linear()
+    var xscale = d3.scale.linear()  //d3.v4: d3.scaleLinear()
 	.domain([0,dlabels.length-1])
 	.range([0,width])
 	.clamp(true);
 
-    var brush = dbrush = d3.svg.brush()
-	.x(xscale)
+    var brush = dbrush = d3.svg.brush()		//d3.v4: d3.brushX() 
+	.x(xscale)				//d3.v4: ???
 	.extent([0, 0])
-	.on("brushstart", dcpOnBrushStart)
-	.on("brush", dcpOnBrush)
-	.on("brushend", dcpOnBrushEnd)
+	.on("brushstart", dcpOnBrushStart)	//d3.v4: ???
+	.on("brush", dcpOnBrush)		//d3.v4: ???
+	.on("brushend", dcpOnBrushEnd)		//d3.v4: ???
     ;
 
 
@@ -860,6 +863,7 @@ function dcpBrushSlider(dlabels, selector, opts) {
 	.call(d3.svg.axis()
 	      .scale(xscale)
 	      .orient("bottom")
+	      //--^ d3.v4: d3.axisBottom(xscale)
 	      .tickSize(12)
 	      .tickPadding(3)
 	      .ticks(dlabels.length-1)
@@ -915,9 +919,9 @@ function dcpBrushSlider(dlabels, selector, opts) {
 
     var x0 = 0; //xscale(dlabels[0]);
     slider
-	.call(brush.event)
+	.call(brush.event)		//-- d3.v4: ???
 	.call(brush.extent([x0,x0]))
-	.call(brush.event);
+	.call(brush.event);		//-- d3.v4: ???
 
     //-- mouse events: hover
     svg
@@ -1811,6 +1815,7 @@ function collide(node, canvasWidth, canvasHeight) {
 var dcpCloudFont = {family:"Impact",weight:"normal",style:"normal"};
 //var dcpCloudFont = {family:"Arial",weight:"bold",style:"normal"};
 //var dcpCloudFont = {family:"Arial Black",weight:"bold",style:"normal"};
+//var dcpCloudFont = {family:"sans-serif",weight:"bold",style:"normal"};
 function dcpFormatCloud(data, jqXHR) {
     //-- parse data
     if ( !(data = dcpParseFlat(data,{mode:"cloud"})) ) { return; }
@@ -1827,7 +1832,7 @@ function dcpFormatCloud(data, jqXHR) {
     var psvg = d3.select("#d3body")
 	.classed("d3chart",true)
 	.attr("title","Tag-cloud for selected date-slice. Click a word to display details.")
-	//.style("background","#333")
+    //.style("background","#333")
 	.append("svg")
 	.attr("width", width)
 	.attr("height", height)
@@ -1839,14 +1844,20 @@ function dcpFormatCloud(data, jqXHR) {
 
 //--------------------------------------------------------------
 // d3: cloud: setup / rescale
+
+//-- random seed (d3.cloud still isn't deterministic even with constant seed!)
+var seed = Math.floor(Math.random() * 65535);
+var xrandom = function() { var x = Math.sin(seed++) * 10000; return x - Math.floor(x); }
+xrandom = Math.random;
+
 var dcpCloudScale = 1;
 var dcpCloudSizeMax = null;
 function dcpCloudSetup(psvg) {
     if (psvg==null)  { psvg=d3.select("svg.d3cloud"); }
 
-    var seed = 1; //-- random seed (debug; don't use 0)
-    //var xrandom = function() { var x = Math.sin(seed++) * 10000; return x - Math.floor(x); }
-    xrandom = Math.random;
+    //xrandom = Math.random;
+    //seed = 42;
+    //debug_log("random seed = " + seed);
 
     //-- size-scaling: re-populate item.size; item.maxSize
     if (dcpCloudSizeMax==null) {
@@ -1956,6 +1967,7 @@ function dcpCloudEnd(placedWords,bounds) {
     placedWords.forEach(function(d) { d.onScreen=true; });
 
     //dcpCloudDebug(); return; //-- DEBUG
+    //dcpCloudDebug(0,'linear',0,true);  return; //-- we don't get collisions if we do dcpCloudDebug() before dcpCouldInterp(): why???
 
     //-- draw data
     dcpCloudInterp(0,1000,"cubic-in-out");
@@ -1978,7 +1990,7 @@ function dcpCloudSnap(value) {
 //--------------------------------------------------------------
 // d3: cloud: update (interp)
 function dcpCloudInterp(value,dur,easeby) {
-    if (dcloud==null) return; //-- not defined yet
+    if (dcloud==null) return; //-- not defined (yet)
     if (value==null) { value = dcur; }
     if (dur==null) { dur = 100; }
     if (easeby==null) { easeby = "linear"; }
@@ -1993,7 +2005,7 @@ function dcpCloudInterp(value,dur,easeby) {
     function tween_font_size(d) {
 	var interp = d3.interpolateRound(d.curSize, ~~dcpItemSize(d,dcur));
 	return function(t) {
-	    $(this).css("font-size",(d.curSize=Math.max(0,interp(t)))+"px");
+	    $(this).css("font-size",(d.curSize=Math.max(dcpSizeRange[0],interp(t)))+"px");
 	};
     };
 
@@ -2011,7 +2023,7 @@ function dcpCloudInterp(value,dur,easeby) {
 	.attr("class","node")
 	.attr("transform", function(d) { return "translate(" + [d.x, d.y] + ")rotate(" + d.rotate + ")"; })
 	.on("click", d3InfoPopup);
-
+    
     //-- enter: title
     entered.append("title");
 
@@ -2019,13 +2031,14 @@ function dcpCloudInterp(value,dur,easeby) {
     entered.append("text")
     //-- enter: text: constant attributes
 	.text(function(d) { return d.text; })
+        //-- these get set by css [but we might need them for svg export?]
+        .attr("text-anchor","middle")
 	.style("font-family",dcpCloudFont.family)
 	.style("font-weight",dcpCloudFont.weight)
 	.style("font-style",dcpCloudFont.style)
 	.style("fill",dcpMinColor())
 	.attr("opacity",0)
-	//.style("font-size",function(d) { (d.curSize=dcpSizeRange[0])+"px"; })
-	.style("font-size",function(d) { (d.curSize=0)+"px"; })
+	.style("font-size",function(d) { (d.curSize=dcpSizeRange[0])+"px"; }) //-- use dcpSizeRange[0] rather than 0 minimum size; 0 --> collisions!
     ;
 
     //-- ENTER+UPDATE: update old or new nodes
@@ -2060,10 +2073,13 @@ function dcpCloudInterp(value,dur,easeby) {
 
 //--------------------------------------------------------------
 // d3: cloud: debug (show all at max size)
-function dcpCloudDebug(dur,easeby) {
+function dcpCloudDebug(dur,easeby,opacity,doInterp) {
     if (dcloud==null) return; //-- not defined yet
     if (dur==null) { dur = 1000; }
     if (easeby==null) { easeby = "linear"; }
+    if (opacity==null) { opacity = 1; }
+    if (doInterp==null) { doInterp = false; }
+    debug_log("dcpCloudDebug("+dur+","+","+easeby+","+opacity+","+doInterp+")");
 
     var ditems = items;
 
@@ -2074,7 +2090,7 @@ function dcpCloudDebug(dur,easeby) {
 	};
     };
 
-    var nodes = d3.select("#profileDataChart > svg > g")
+    var nodes = d3.select("svg.d3cloud > g")
 	.selectAll(".node")
 	.data(ditems, function(d) { return d.id; });
 
@@ -2090,16 +2106,14 @@ function dcpCloudDebug(dur,easeby) {
 	.on("click", d3InfoPopup);
     entered
 	.append("title")
-	.text(function(d) {
-	    return (d.label + " ~ " + dcpItemScore(d,dcur)
-		    + (user_query.debug ? (": " + JSON.stringify({id:d.id, value:dcpItemValue(d,dcur), avalue:dcpItemAbsValue(d,dcur), size:~~dcpItemSize(d,dcur)})) : '')
-		   );
-	});
+    ;
+    
     //-- enter: text
     entered.append("text")
     //-- enter: text: constant attributes
-	.attr("text-anchor","middle")
 	.text(function(d) { return d.text; })
+        //-- these get set by css [but we might need them for svg export?]
+        .attr("text-anchor","middle")
 	.style("font-family",dcpCloudFont.family)
 	.style("font-weight",dcpCloudFont.weight)
 	.style("font-style",dcpCloudFont.style)
@@ -2109,13 +2123,21 @@ function dcpCloudDebug(dur,easeby) {
     ;
 
     //-- ENTER+UPDATE: update old or new nodes
+    var ntrans=0;
     nodes.selectAll("text")
 	.transition()
 	.duration(dur)
 	.ease(easeby)
-	.attr("opacity", 1)
+	.attr("opacity", opacity)
 	.style("fill", function(d) { return heatcolorf(d3.max(isDiff ? d.avalue : d.value)); })
 	.tween("font-size",tween_font_size)
+    //
+    	.each("start", function() { ntrans++; })
+	.each("end",function() {
+	    if (--ntrans === 0 && doInterp) {
+		dcpCloudInterp(0,1000,"cubic-in-out");
+	    }
+	})
     ;
 
     //-- EXIT: remove old elements as needed
@@ -2123,7 +2145,8 @@ function dcpCloudDebug(dur,easeby) {
 	.transition()
 	.duration(dur)
 	.ease(easeby)
-	.remove();
+	.remove()
+    ;
     exited.selectAll("text")
 	.attr("opacity", 0)
 	.style("fill", dcpMinColor())
